# CISE Testing Framework in TTCN-3

## Introduction
This repositories contains the test specifications and test adapter code for CISE API Conformance Testing in TTCN-3.

More information and dowload of the standard at [here](https://portal.etsi.org/webapp/WorkProgram/Report_WorkItem.asp?WKI_ID=66165).

For more information on ETSI ISG Commom Data MODEL (CDM), visit [here](https://www.etsi.org/technologies/maritime).

## Contact information

Email at `cti_support` at `etsi` dot `org`.

## License

Unless specified otherwise, the content of this repository and the files contained are released under the BSD-3-Clause license.
See the attached LICENSE file or visit https://forge.etsi.org/legal-matters.

## Standard links
- CISE standards, accessible [here](https://www.etsi.org/standards#page=1&search=CISE&title=1&etsiNumber=1&content=1&version=0&onApproval=1&published=1&historical=1&startDate=1988-01-15&endDate=2022-11-07&harmonized=0&keyword=&TB=&stdType=&frequency=&mandate=&collection=&sort=1).

## RFC links
- RFC 7159 - The JavaScript Object Notation (JSON) Data Interchange Format, accessible [here](https://tools.ietf.org/html/rfc7159.html)

NOTE All drafts can be found in the 'IETF XML Registry', accessible [here](https://www.iana.org/assignments/xml-registry/xml-registry.xhtml)

## Download required libraries

This projects requires the LibCommon and Titan libraries to compile. The libraries location are git submodules of the current repository.

Once you have cloned the present repository, in order to install the libraries you want to execute:

    git submodule init
    git submodule update

Alternatively, you may clone the project together with the required libraries with the following command:

    git clone --recurse-submodules <URL to the repository>

# Installation

The CISE Test System builds and tests regularly on the following platforms:

  - Linux (Ubuntu)

Note: The [OpenSSL](https://www.openssl.org) version > 1.1.x is also required.

## How to do it?

They are two different methods:
- Using [Vagrant](https://www.vagrantup.com/)
- Using [Docker](https://www.docker.com/)

How to choose one of these methods is depending of your host system.

NOTE: In all case, if you want to setup an continuous integration process (e.g. Jenkins), Docker is the best choice. 


### The host system is Windows
The both methods require a virtual machine. You can use either VirtualBox or WMware.
In this case, the easiest way is to use Vagrant.


### The host system is Linux

Vagrant requires a virtual machine. You can use either VirtualBox or WMware.
Docker does not need a virtual machine, so it is the more efficant way.

The following clauses describes four ways to build and deploy the CISE  Test System:
- [Using TITAN compiler running on a Docker image](#using-titan-compiler-on-a-docker-image)
- [Using development Docker image](#using-development-docker-image)
- [Using Vagrant](#using-vagrant)
- [From scratch](#from-scratch)


## Using TITAN compiler running on a Docker image

In this configuration, TITAN compiler is located on a Docker image and the sources and the outputs are located on the host.

Pre-requisites on your host machine:
- Install Docker

Procedure on the host machine:
- Open a Terminal
- Clone the ETSI CISE Test System
- [Install Test System certificates](#generate-certificates-for-tls-and-cise-security-support)

```sh
$ git clone --recurse-submodules --branch stf637 https://forge.etsi.org/rep/cdm/cise-data-model ./cise-data-model
$ cd /home/etsi/dev/cise-data-model/titan-test-system-framework
$ git checkout devel
$ cd /home/etsi/dev/cise-data-model/titan-test-system-framework/ttcn/LibHttp
$ ln -sf module_cise.mk module.mk
```

- Patch the LibCommon

```sh
$ cd /home/etsi/dev/cise-data-model
$ cp ./ttcn/patch_lib_common_titan/module.mk ./ttcn/LibCommon/
$ cp ./ttcn/patch_lib_common_titan/ttcn/*.ttcn ./ttcn/LibCommon/ttcn
```

- From the ETSI CISE Test System root directory, build the Docker image executing the following commands:

```sh
$ cd /home/etsi/dev/cise-data-model/virtualization/docker
$ docker build --no-cache --tag alpine-cise -f Dockerfile --force-rm .
$ docker images
```

To build the CISE Test Suite, execute the following command:

```sh
$ ./docker-run.sh build
```

Possble other options are 'clean' to remove all the build outputs or 'rebuild' to force a build of the Test Suite after a 'clean'.

To retrieve the list of the available test cases, execute the following command:

```sh
$ ./docker-run.sh list
```

Before to execute the CISE Test Suite, prepare, edit and update the configuration file:

```sh
$ ln -sf ../../etc/AtsCise/AtsCise_Cnit.cf_ ../../etc/AtsCise/AtsCise.cfg
$ vi ../../etc/AtsCise/AtsCise.cfg # To update it
```

The paramaters to be updated are:
- PICS_CISE_CERTIFICATES_DB_PATH
- PICS_CISE_TS_CERTIFICATE
- PICS_CISE_TS_SIGNING_PRIVATE_KEY
- PICS_CISE_TS_SIGNING_PRIVATE_PASSWD
- PICS_HEADER_HOST
- system.httpPort.params
- system.httpPort_notif.params

To execute the CISE Test Suite, execute the following command:

```sh
$ ./docker-run.sh run
```

<b>Notes</b>:
- The CISE Test System is listening on port 30181
- Updating the file etc/AtsCise.cfg is about:
. Selecting the test(s) to be executed
. Updating value of PICs and PIXITs
. Updating HTTP port setting


## Using development Docker image

In this configuration, the TITAN compiler, the sources and the outputs are located on the Docker image.

Pre-requisites on your host machine:
- Install Docker

Procedure for a Windows host machine:
- On your host machine, open a the Docker Quickstart Terminal
- On your host machine, clone the ETSI CISE Test System. NOTE that only Docker folder and .jenkins.sh script file are required
- From the ETSI CISE Test System root directory, execute the following commands:

```sh
$ ./.jenkins.sh
...
```

NOTE The creation and the installations will take some time to achieve
- Start the container

```sh
$ ./docker/run-container.sh
...
```

- Switch to the next clause (Usage)


## Using Vagrant

Pre-requisites on your host machine:
- Install [Virtualbox](https://www.virtualbox.org/manual/ch01.html)
- Install [Vagrant](https://www.vagrantup.com/intro/getting-started/)
- Install Vagrant plugin vagrant-vbguest
- Credentials to access [ETSI forge](https://forge.etsi.org/gitlab/users/sign_in)
    - Set the environment variable USERNAME to your ETSI EOL account user name
    - Set the environment variable PASSWORD to your ETSI EOL account password

Procedure:
- On your host machine, open a command line session (PuTTY, DOS window...)
- From the ETSI CISE Test System, clone the Vagrant folder
- In the file Vagrantfile, modify the tag config.vm.provision replacing <username> & <password> strings by your ETSI credentials
- In the Vagrant folder, execute the following commands:

```sh
$ vagrant up --provider virtualbox --provision
...
```

NOTE The creation and the installations will take some time to achieve
- Stop vagrant virtual machine

```sh
$ vagrant halt
...
```

- Update the file 'Vagrantfile' to match with your networks configuration
- Re-start the vagrant virtual machine and log to to the machine

```sh
$ vagrant up
...
$ vagrant ssh
```

- Switch to the next clause (Usage)

NOTE The user password is vagrant.


## From scratch

Pre-requisites:
- Install Virtualbox

Procedure:
- Install a new Linux Virtual machine (Mint, Debian...)
- Update your system with the latest version of kernel and security packages
- Install the following packages (According to the Linux chosen, the package naming can be different)
    autoconf
    bison
    build-essential
    cmake
    curl
    dos2unix
    doxygen
    emacs
    expect
    flex
    g++:latest
    gcc:latest
    graphviz
    gdb 
    git-core
    gnutls-bin
    libglib2.0-dev
    libpcap-dev
    libgcrypt-dev
    libncurses5-dev
    libssl-dev
    libtool-bin
    libtool
    libwireshark-dev
    libxml2-dev
    lsof
    ntp
    pkg-config
    qt5-default
    qtmultimedia5-dev
    libqt5svg5-dev
    subversion
    sudo
    sshpass
    tcpdump
    texlive-font-utils
    tshark
    valgrind
    vim
    vsftpd
    xutils-dev 
    tree
    tzdata
    unzip
    wget
    xsltproc
- In your home directory, create the following folders: 
    - $HOME/frameworks, 
    - $HOME/dev
    - $HOME/lib
- In $HOME/frameworks, build the following package:
    - Eclipse IDE for C/C++ Developers, according the procedure specified [here](https://www.eclipse.org/cdt/)
    - TITAN, according the procedure specified [here](https://github.com/eclipse/titan.core)
    - Import the TITAN plugin into your Eclipse IDE, according the procedure specified [here](https://github.com/eclipse/titan.core)
- Clone the ETSI CISE Test System into $HOME/dev folder

```sh
$ git clone --recurse-submodules --branch --branch stf637 https://forge.etsi.org/rep/cdm/cise-data-model ./cise-data-model
$ cd /home/etsi/dev/cise-data-model/titan-test-system-framework
$ git checkout devel
$ cd /home/etsi/dev/cise-data-model/titan-test-system-framework/ttcn/LibHttp
$ ln -sf module_cise.mk module.mk
```

- Apply patches

```sh
$ cd /home/etsi/dev/cise-data-model
$ cp ./ttcn/patch_lib_common_titan/module.mk ./ttcn/LibCommon/
$ cp ./ttcn/patch_lib_common_titan/ttcn/*.ttcn ./ttcn/LibCommon/ttcn/
```

- Update your default environment with the content of the script $HOME/dev/cise-data-model/scripts/devenv.bash.ubuntu

- Switch to the next clause (Usage)


# Generate certificates for TLS and Cise security support 

This clause describes how generate certificates desrived from Let's encrypt certificate. These certificate will be used for TS mutual authentication and for Cise Security tests.
<b>NOTE:</b>
- Certficates shall be renewed every 3 months
- The port 80 shall be vailable for standalone validation

Pre-requisites:
- You need to install python and [certbot](https://manpages.ubuntu.com/manpages/impish/en/man1/certbot.1.html).

To generate certifcates, execute the following command:

```sh
$ export REQUESTS_CA_BUNDLE=$(dirname `python -c "import certifi; print(certifi.where())"`)
$ mkdir -p $HOME/var/ssl
$ sudo certbot certonly --debug --cise-tls-cert --config-dir $HOME/var/ssl --work-dir $HOME/var/ssl  --logs-dir $HOME/var/ssl --standalone --agree-tos --email <your email> -d <your domain> -w $HOME/var/ssl/
$ sudo certbot certonly --debug --cise-test-cert --config-dir $HOME/var/ssl --work-dir $HOME/var/ssl  --logs-dir $HOME/var/ssl --standalone --agree-tos --email <your email> -d <your domain> -w $HOME/var/ssl/
```

<b>NOTE:</b> For testing certificate generation and renewal, use the certbot's --dry-run option.


# Usage

This clause describes how to compile and execute an Abstract Test Suite.


Pre-requisites:
- Your machine is installed following one of the installation method describes in the previous clause
- Your are logged as 'etsi' or 'vagrant' user

Procedure using Eclipse TITAN:
- Start eclipse using a new workspace, (e.g. with the name workspace_titan)
- Download and follow the steps to install Eclipse plugins for TITAN, accessible [here]{https://www.eclipse.org/downloads/download.php?file=/titan/Eclipse_installationguide.pdf}
- Open the workspace_titan
- Create a new TITAN project (e.g. cise-data-model)
<TODO>

Procedure in TITAN command line:
- Open several SSH session (PuTTY...)
- Change to the directory ~/dev/cise-data-model/src/AtsCise/objs
- Build the test suite AtsCise

```sh
$ cd ~/dev/cise-data-model/ && export ATS=AtsCise
$ make
...
```

- Edit the file ../etc/AtsCise.cfg
- Update the following fields:
    - PICS_HEADER_HOST, the HTTP host header value
    - system.httpPort.params
- To run the test suite, execute the following command:

```sh
$ cd ~/dev/cise-data-model/scripts/
$ ./run-all.bash
...
```

- The log files are located in ../logs folder. You can edit them using any editor or using the Eclipse TITAN log plugins


# How to Report a Bug

The ETSI CISE Test System is under constant development, so it is possible that you will
encounter a bug while using it. Please report bugs at cti_support at etsi dot org.
