#include "http_codec_cise.hh"

#include "loggers.hh"

bool http_codec_cise::encode_body_xml(const LibHttp__XmlMessageBodyTypes::XmlBody &p_xml_body, OCTETSTRING &p_encoding_buffer, const std::string& p_content_type) {
  loggers::get_instance().log(">>> http_codec_cise::encode_body_xml");

  std::map<std::string, std::unique_ptr<codec_gen<Record_Type, Record_Type> > >::const_iterator it;
  bool processed = false;
  loggers::get_instance().log("http_codec_cise::encode_body_xml: Content-Type:'%s'", p_content_type.c_str());
  if (p_content_type.find("xml") != std::string::npos) {
    it = _codecs.find("xml"); // TODO Use params
    if (it != _codecs.cend()) {
      loggers::get_instance().log("http_codec_cise::encode_body_xml: Call 'xml_codec'");
      _codecs["xml"]->encode((Record_Type&)p_xml_body, p_encoding_buffer); // TODO Use params
      processed = true;
    }
  } // TODO Add new HTTP message codec here
  if (!processed) {
    loggers::get_instance().warning("http_codec_cise::encode_body_xml: Unsupported HTTP codec, use raw field as default");
    p_encoding_buffer = OCTETSTRING(0, nullptr);
  }

  return true;
}

bool http_codec_cise::decode_body_xml(const OCTETSTRING &p_data, LibHttp__XmlMessageBodyTypes::XmlBody &p_body, const std::string& p_content_type, params* p_params) {
  loggers::get_instance().log_msg(">>> http_codec_cise::decode_body_xml: ", p_data);

  if (_codecs["xml"].get() != nullptr) {
    p_body.raw().set_to_omit();
    loggers::get_instance().log("http_codec_cise::decode_body: Call 'xml_codec'");
    if (_codecs["xml"]->decode(p_data, (Record_Type&)p_body, p_params) == -1) {
      loggers::get_instance().warning("http_codec_cise::decode_body: Failed to decode JSON message");
      p_body.raw() = oct2char(p_data);
    } else {
      loggers::get_instance().log_msg("http_codec_cise::decode_body: Decoded message:", p_body.msg());
      p_body.raw() = oct2char(p_data);
    }
  }

  return true;
}

bool http_codec_cise::encode_body_json(const LibHttp__JsonMessageBodyTypes::JsonBody &p_json_body, OCTETSTRING &p_encoding_buffer, const std::string& p_content_type) {
  loggers::get_instance().log_msg(">>> http_codec_cise::encode_body_json: ", p_json_body);
  
  std::map<std::string, std::unique_ptr<codec_gen<Record_Type, Record_Type> > >::const_iterator it;
  bool processed = false;
  loggers::get_instance().log("http_codec_cise::encode_body_json: Content-Type:'%s'", p_content_type.c_str());
  if (p_content_type.find("json") != std::string::npos) {
    it = _codecs.find("json"); // TODO Use params
    if (it != _codecs.cend()) {
      loggers::get_instance().log("http_codec_cise::encode_body_json: Call 'json_codec'");
      _codecs["json"]->encode((Record_Type&)p_json_body, p_encoding_buffer); // TODO Use params
      processed = true;
    }
  } // TODO Add new HTTP message codec here
  if (!processed) {
    loggers::get_instance().warning("http_codec_cise::encode_body_json: Unsupported HTTP codec, use raw field as default");
    p_encoding_buffer = OCTETSTRING(0, nullptr);
  }

  return true;
}

bool http_codec_cise::decode_body_json(const OCTETSTRING &p_data, LibHttp__JsonMessageBodyTypes::JsonBody &p_json_body, const std::string& p_content_type, params* p_params) {
  loggers::get_instance().log(">>> http_codec_cise::decode_body_json: content: %s", p_content_type.c_str());
  loggers::get_instance().log_msg(">>> http_codec_cise::decode_body_json: data: ", p_data);

  if (_codecs["json"].get() != nullptr) {
    loggers::get_instance().log("http_codec_cise::decode_body_json: Call 'json_codec'");
    if (_codecs["json"]->decode(p_data, (Record_Type&)p_json_body, p_params) == -1) {
      loggers::get_instance().warning("http_codec_cise::decode_body_json: Failed to decode JSON message");
      p_json_body.raw() = CHARSTRING(p_data.lengthof(), (char*)static_cast<const unsigned char*>(p_data));
    } else {
      loggers::get_instance().log_msg("http_codec_cise::decode_body_json: Decoded message: ", p_json_body);
    }
  } else {
      loggers::get_instance().error("http_codec_cise::decode_body_json: Codec not found");
      return false;
  }

  return true;
}
