#!/usr/bin/env bash

#set +vx
set -euo pipefail

usage() {
	echo "No command specified. Available commands:"
	for i in $(echo $commands | sed "s/,/ /g")
	do
		echo "  $i"
	done
	echo "For build,clean,list,modulepar,rebuild,build-run and run"
	echo "e.g. ./virtualization/docker/docker-run.sh run # To execute test suite"
}

cli="${GEN_DIR}/bin/AtsCise"

commands="help,build,clean,ip,list,modulepar,ping,rebuild,build-run,run,version"

if [ $# == 0 ]; then
	echo "No command specified. Available commands:"
	for i in $(echo $commands | sed "s/,/ /g")
	do
		echo "  $i"
	done
	exit 1
fi

case "$1" in
  build)
    cd ${GEN_DIR} && make
    ;;
  clean)
    cd ${GEN_DIR} && make clean
    ;;
  ip)
    ip address
    ;;
  list)
    ${cli} -l
    ;;
  modulepar)
    ${cli} -p
    ;;
  ping)
    sudo ping $2
    ;;
  rebuild)
    cd ${GEN_DIR} && rm -fr ./bin ./build ; make
    ;;
  build-run)
    cd ${GEN_DIR}/scripts && ./run_all.bash 0
    ;;
  run)
    cd ${GEN_DIR}/scripts && ./run_all.bash
    ;;
  version)
    compiler -v
    ;;
esac

exit 0
